@@ -75,7 +75,9 @@ def __virtual__():
             log.debug('Configuration option \'providers:pkg\' is set to '
                 '\'pkgng\', using \'pkgng\' in favor of \'freebsdpkg\'.')
             return __virtualname__
-    return False
+    return (False,
+            'The pkgng execution module cannot be loaded: only available '
+            'on FreeBSD 10 or FreeBSD 9 with providers.pkg set to pkgng.')
 
 
 def _pkg(jail=None, chroot=None):
@@ -83,11 +85,11 @@ def _pkg(jail=None, chroot=None):
     Returns the prefix for a pkg command, using -j if a jail is specified, or
     -c if chroot is specified.
     '''
-    ret = 'pkg'
+    ret = ['pkg']
     if jail:
-        ret += ' -j {0!r}'.format(jail)
+        ret.extend(['-j', jail])
     elif chroot:
-        ret += ' -c {0!r}'.format(chroot)
+        ret.extend(['-c', chroot])
     return ret
 
 
@@ -103,7 +105,7 @@ def _get_version(name, results):
     ``pkg search`` will return all packages for which the pattern is a match.
     Narrow this down and return the package version, or None if no exact match.
     '''
-    for line in results.splitlines():
+    for line in salt.utils.itertools.split(results, '\n'):
         if not line:
             continue
         try:
@@ -237,12 +239,11 @@ def refresh_db(jail=None, chroot=None, force=False):
 
             salt '*' pkg.refresh_db force=True
     '''
-    opts = ''
+    cmd = _pkg(jail, chroot)
+    cmd.append('update')
     if force:
-        opts += ' -f'
-    return __salt__['cmd.retcode'](
-        '{0} update{1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False) == 0
+        cmd.append('-f')
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 # Support pkg.update to refresh the db, since this is the CLI usage
@@ -282,6 +283,7 @@ def latest_version(*names, **kwargs):
     else:
         quiet = False
 
+    cmd_prefix = _pkg(jail, chroot) + ['search']
     for name in names:
         # FreeBSD supports packages in format java/openjdk7
         if '/' in name:
@@ -377,12 +379,11 @@ def list_pkgs(versions_as_list=False,
 
     ret = {}
     origins = {}
-    cmd = '{0} info -ao'.format(_pkg(jail, chroot))
     out = __salt__['cmd.run_stdout'](
-            cmd,
-            python_shell=False,
-            output_loglevel='trace')
-    for line in out.splitlines():
+        _pkg(jail, chroot) + ['info', '-ao'],
+        output_loglevel='trace',
+        python_shell=False)
+    for line in salt.utils.itertools.split(out, '\n'):
         if not line:
             continue
         try:
@@ -484,16 +485,13 @@ def stats(local=False, remote=False, jail=None, chroot=None):
         opts += 'l'
     if remote:
         opts += 'r'
-    if opts:
-        opts = '-' + opts
 
-    res = __salt__['cmd.run'](
-        '{0} stats {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
-    )
-    res = [x.strip("\t") for x in res.split("\n")]
-    return res
+    cmd = _pkg(jail, chroot)
+    cmd.append('stats')
+    if opts:
+        cmd.append('-' + opts)
+    out = __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
+    return [x.strip('\t') for x in salt.utils.itertools.split(out, '\n')]
 
 
 def backup(file_name, jail=None, chroot=None):
@@ -529,12 +527,12 @@ def backup(file_name, jail=None, chroot=None):
 
             salt '*' pkg.backup /tmp/pkg chroot=/path/to/chroot
     '''
-    res = __salt__['cmd.run'](
-        '{0} backup -d {1!r}'.format(_pkg(jail, chroot), file_name),
-        python_shell=False,
-        output_loglevel='trace'
+    ret = __salt__['cmd.run'](
+        _pkg(jail, chroot) + ['backup', '-d', file_name],
+        output_loglevel='trace',
+        python_shell=False
     )
-    return res.split('...')[1]
+    return ret.split('...')[1]
 
 
 def restore(file_name, jail=None, chroot=None):
@@ -571,9 +569,9 @@ def restore(file_name, jail=None, chroot=None):
             salt '*' pkg.restore /tmp/pkg chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} backup -r {1!r}'.format(_pkg(jail, chroot), file_name),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot) + ['backup', '-r', file_name],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -607,9 +605,9 @@ def audit(jail=None, chroot=None):
             salt '*' pkg.audit chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} audit -F'.format(_pkg(jail, chroot)),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot) + ['audit', '-F'],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -760,7 +758,6 @@ def install(name=None,
         return {}
 
     opts = 'y'
-    repo_opts = ''
     if salt.utils.is_true(orphan):
         opts += 'A'
     if salt.utils.is_true(force):
@@ -775,16 +772,10 @@ def install(name=None,
         opts += 'q'
     if salt.utils.is_true(reinstall_requires):
         opts += 'R'
-    if fromrepo:
-        repo_opts += 'r {0}'.format(fromrepo)
     if salt.utils.is_true(regex):
         opts += 'x'
     if salt.utils.is_true(pcre):
         opts += 'X'
-    if opts:
-        opts = '-' + opts
-    if repo_opts:
-        repo_opts = '-' + repo_opts
 
     old = list_pkgs(jail=jail, chroot=chroot)
 
@@ -806,19 +797,41 @@ def install(name=None,
             else:
                 targets.append('{0}-{1}'.format(param, version_num))
 
-    cmd = '{0} {1} {2} {3} {4}'.format(
-        _pkg(jail, chroot), pkg_cmd, repo_opts, opts, ' '.join(targets)
-    )
+    cmd = _pkg(jail, chroot)
+    cmd.append(pkg_cmd)
+    if fromrepo:
+        cmd.extend(['-r', fromrepo])
+    if opts:
+        cmd.append('-' + opts)
+    cmd.extend(targets)
 
     if pkg_cmd == 'add' and salt.utils.is_true(dryrun):
         # pkg add doesn't have a dryrun mode, so echo out what will be run
-        return cmd
+        return ' '.join(cmd)
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
 
-    __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
     __context__.pop(_contextkey(jail, chroot), None)
     __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
     new = list_pkgs(jail=jail, chroot=chroot)
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def remove(name=None,
@@ -959,17 +972,36 @@ def remove(name=None,
         opts += 'x'
     if salt.utils.is_true(pcre):
         opts += 'X'
-    if opts:
-        opts = '-' + opts
 
-    cmd = '{0} delete {1} {2}'.format(
-        _pkg(jail, chroot), opts, ' '.join(targets)
+    cmd = _pkg(jail, chroot)
+    cmd.append('delete')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.extend(targets)
+
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
-    __salt__['cmd.run'](cmd, python_shell=False, output_loglevel='trace')
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop(_contextkey(jail, chroot), None)
     __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
     new = list_pkgs(jail=jail, chroot=chroot, with_origin=True)
-    return salt.utils.compare_dicts(old, new)
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 # Support pkg.delete to remove packages, since this is the CLI usage
 delete = salt.utils.alias_function(remove, 'delete')
@@ -1063,22 +1095,28 @@ def upgrade(*names, **kwargs):
     if opts:
         opts = '-' + opts
 
+    cmd = _pkg(jail, chroot)
+    cmd.append('upgrade')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.extend(names)
+
     old = list_pkgs()
-    call = __salt__['cmd.run_all'](
-        '{0} upgrade {1} {2}'.format(_pkg(jail, chroot), opts, ' '.join(names)),
-        python_shell=False,
-        output_loglevel='trace'
-    )
+    call = __salt__['cmd.run_all'](cmd,
+                                   output_loglevel='trace',
+                                   python_shell=False,
+                                   redirect_stderr=True)
+
     if call['retcode'] != 0:
         ret['result'] = False
-        if 'stderr' in call:
-            ret['comment'] += call['stderr']
-        if 'stdout' in call:
-            ret['comment'] += call['stdout']
-    else:
-        __context__.pop('pkg.list_pkgs', None)
-        new = list_pkgs()
-        ret['changes'] = salt.utils.compare_dicts(old, new)
+        if call['stdout']:
+            ret['comment'] = call['stdout']
+
+    __context__.pop(_contextkey(jail, chroot), None)
+    __context__.pop(_contextkey(jail, chroot, prefix='pkg.origin'), None)
+    new = list_pkgs()
+    ret['changes'] = salt.utils.compare_dicts(old, new)
+
     return ret
 
 
@@ -1095,9 +1133,9 @@ def clean(jail=None, chroot=None):
         salt '*' pkg.clean chroot=/path/to/chroot
     '''
     return __salt__['cmd.run'](
-        '{0} clean'.format(_pkg(jail, chroot)),
-        python_shell=False,
-        output_loglevel='trace'
+        _pkg(jail, chroot) + ['clean'],
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -1124,12 +1162,15 @@ def autoremove(jail=None, chroot=None, dryrun=False):
         opts += 'n'
     else:
         opts += 'y'
+
+    cmd = _pkg(jail, chroot)
+    cmd.append('autoremove')
     if opts:
-        opts = '-' + opts
+        cmd.append('-' + opts)
     return __salt__['cmd.run'](
-        '{0} autoremove {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -1200,13 +1241,15 @@ def check(jail=None,
         opts += 'r'
     if checksum:
         opts += 's'
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot)
+    cmd.append('check')
+    if opts:
+        cmd.append('-' + opts)
     return __salt__['cmd.run'](
-        '{0} check {1}'.format(_pkg(jail, chroot), opts),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -1263,12 +1306,16 @@ def which(path, jail=None, chroot=None, origin=False, quiet=False):
         opts += 'q'
     if origin:
         opts += 'o'
+
+    cmd = _pkg(jail, chroot)
+    cmd.append('which')
     if opts:
-        opts = '-' + opts
+        cmd.append('-' + opts)
+    cmd.append(path)
     return __salt__['cmd.run'](
-        '{0} which {1} {2}'.format(_pkg(jail, chroot), opts, path),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -1450,13 +1497,16 @@ def search(name,
         opts += 'o'
     if prefix:
         opts += 'p'
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot)
+    cmd.append('search')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} search {1} {2}'.format(_pkg(jail, chroot), opts, name),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -1574,13 +1624,10 @@ def fetch(name,
             salt '*' pkg.fetch <package name> depends=True
     '''
     opts = ''
-    repo_opts = ''
     if fetch_all:
         opts += 'a'
     if quiet:
         opts += 'q'
-    if fromrepo:
-        repo_opts += 'r {0}'.format(fromrepo)
     if glob:
         opts += 'g'
     if regex:
@@ -1591,17 +1638,18 @@ def fetch(name,
         opts += 'L'
     if depends:
         opts += 'd'
-    if opts:
-        opts = '-' + opts
-    if repo_opts:
-        repo_opts = '-' + repo_opts
 
+    cmd = _pkg(jail, chroot)
+    cmd.extend(['fetch', '-y'])
+    if fromrepo:
+        cmd.extend(['-r', fromrepo])
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} fetch -y {1} {2} {3}'.format(
-            _pkg(jail, chroot), opts, repo_opts, name
-        ),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
 
 
@@ -1662,11 +1710,14 @@ def updating(name,
         opts += 'd {0}'.format(filedate)
     if filename:
         opts += 'f {0}'.format(filename)
-    if opts:
-        opts = '-' + opts
 
+    cmd = _pkg(jail, chroot)
+    cmd.append('updating')
+    if opts:
+        cmd.append('-' + opts)
+    cmd.append(name)
     return __salt__['cmd.run'](
-        '{0} updating {1} {2}'.format(_pkg(jail, chroot), opts, name),
-        python_shell=False,
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
