@@ -4,15 +4,19 @@ Module to provide RabbitMQ compatibility to Salt.
 Todo: A lot, need to add cluster support, logging, and minion configuration
 data.
 '''
-
-# Import Python Libs
 from __future__ import absolute_import
+
+# Import python libs
+import json
 import logging
 import random
 import string
 
-# Import Salt Libs
+# Import salt libs
 import salt.utils
+import salt.utils.itertools
+import salt.ext.six as six
+from salt.exceptions import SaltInvocationError
 from salt.ext.six.moves import range
 from salt.exceptions import CommandExecutionError
 
@@ -59,25 +63,27 @@ def _get_rabbitmq_plugin():
     return rabbitmq
 
 
+def _safe_output(line):
+    '''
+    Looks for rabbitmqctl warning, or general formatting, strings that aren't
+    intended to be parsed as output.
+    Returns a boolean whether the line can be parsed as rabbitmqctl output.
+    '''
+    return not any([
+        line.startswith('Listing') and line.endswith('...'),
+        '...done' in line,
+        line.startswith('WARNING:')
+    ])
+
+
 def _strip_listing_to_done(output_list):
-    '''Conditionally remove non-relevant first and last line,
+    '''
+    Conditionally remove non-relevant first and last line,
     "Listing ..." - "...done".
     outputlist: rabbitmq command output split by newline
     return value: list, conditionally modified, may be empty.
     '''
-
-    # conditionally remove non-relevant first line
-    f_line = ''.join(output_list[:1])
-    if f_line.startswith('Listing') and f_line.endswith('...'):
-        output_list.pop(0)
-
-    # some versions of rabbitmq have no trailing '...done' line,
-    # which some versions do not output.
-    l_line = ''.join(output_list[-1:])
-    if '...done' in l_line:
-        output_list.pop()
-
-    return output_list
+    return [line for line in output_list if _safe_output(line)]
 
 
 def _output_to_dict(cmdoutput, values_mapper=None):
@@ -122,8 +128,10 @@ def list_users(runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run']('rabbitmqctl list_users',
-                              runas=runas)
+    res = __salt__['cmd.run'](
+        ['rabbitmqctl', 'list_users'],
+        runas=runas,
+        python_shell=False)
 
     # func to get tags from string such as "[admin, monitoring]"
     func = lambda string: set([x.strip() for x in string[1:-1].split(',')])
@@ -201,10 +209,10 @@ def add_user(name, password=None, runas=None):
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl add_user {0} {1!r}'.format(name, password),
+        ['rabbitmqctl', 'add_user', name, password],
         output_loglevel='quiet',
-        python_shell=False,
-        runas=runas)
+        runas=runas,
+        python_shell=False)
 
     if clear_pw:
         # Now, Clear the random password from the account, if necessary
@@ -233,9 +241,10 @@ def delete_user(name, runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run']('rabbitmqctl delete_user {0}'.format(name),
-                              python_shell=False,
-                              runas=runas)
+    res = __salt__['cmd.run'](
+        ['rabbitmqctl', 'delete_user', name],
+        python_shell=False,
+        runas=runas)
     msg = 'Deleted'
 
     return _format_response(res, msg)
@@ -254,10 +263,10 @@ def change_password(name, password, runas=None):
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl change_password {0} {1!r}'.format(name, password),
+        ['rabbitmqctl', 'change_password', name, password],
+        runas=runas,
         output_loglevel='quiet',
-        python_shell=False,
-        runas=runas)
+        python_shell=False)
     msg = 'Password Changed'
 
     return _format_response(res, msg)
@@ -275,14 +284,51 @@ def clear_password(name, runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run']('rabbitmqctl clear_password {0}'.format(name),
-                              python_shell=False,
-                              runas=runas)
+    res = __salt__['cmd.run'](
+        ['rabbitmqctl', 'clear_password', name],
+        runas=runas,
+        python_shell=False)
     msg = 'Password Cleared'
 
     return _format_response(res, msg)
 
 
+def check_password(name, password, runas=None):
+    '''
+    .. versionadded:: 2016.3.0
+
+    Checks if a user's password is valid.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' rabbitmq.check_password rabbit_user password
+    '''
+    if runas is None:
+        runas = salt.utils.get_user()
+
+    cmd = ('rabbit_auth_backend_internal:check_user_login'
+        '(<<"{0}">>, [{{password, <<"{1}">>}}]).').format(
+        name.replace('"', '\\"'),
+        password.replace('"', '\\"'))
+
+    res = __salt__['cmd.run'](
+        ['rabbitmqctl', 'eval', cmd],
+        runas=runas,
+        output_loglevel='quiet',
+        python_shell=False)
+    msg = 'password-check'
+
+    _response = _format_response(res, msg)
+    _key = _response.keys()[0]
+
+    if 'invalid credentials' in _response[_key]:
+        return False
+
+    return True
+
+
 def add_vhost(vhost, runas=None):
     '''
     Adds a vhost via rabbitmqctl add_vhost.
@@ -295,9 +341,10 @@ def add_vhost(vhost, runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run']('rabbitmqctl add_vhost {0}'.format(vhost),
-                              python_shell=False,
-                              runas=runas)
+    res = __salt__['cmd.run'](
+        ['rabbitmqctl', 'add_vhost', vhost],
+        runas=runas,
+        python_shell=False)
 
     msg = 'Added'
     return _format_response(res, msg)
@@ -315,9 +362,10 @@ def delete_vhost(vhost, runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run']('rabbitmqctl delete_vhost {0}'.format(vhost),
-                              python_shell=False,
-                              runas=runas)
+    res = __salt__['cmd.run'](
+        ['rabbitmqctl', 'delete_vhost', vhost],
+        runas=runas,
+        python_shell=False)
     msg = 'Deleted'
     return _format_response(res, msg)
 
@@ -335,10 +383,10 @@ def set_permissions(vhost, user, conf='.*', write='.*', read='.*', runas=None):
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl set_permissions -p {0} {1} "{2}" "{3}" "{4}"'.format(
-            vhost, user, conf, write, read),
-        python_shell=False,
-        runas=runas)
+        ['rabbitmqctl', 'set_permissions', '-p',
+         vhost, user, conf, write, read],
+        runas=runas,
+        python_shell=False)
     msg = 'Permissions Set'
     return _format_response(res, msg)
 
@@ -356,9 +404,9 @@ def list_permissions(vhost, runas=None):
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl list_permissions -p {0}'.format(vhost),
-        python_shell=False,
-        runas=runas)
+        ['rabbitmqctl', 'list_permissions', '-p', vhost],
+        runas=runas,
+        python_shell=False)
 
     return _output_to_dict(res)
 
@@ -376,9 +424,9 @@ def list_user_permissions(name, runas=None):
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl list_user_permissions {0}'.format(name),
-        python_shell=False,
-        runas=runas)
+        ['rabbitmqctl', 'list_user_permissions', name],
+        runas=runas,
+        python_shell=False)
 
     return _output_to_dict(res)
 
@@ -399,9 +447,9 @@ def set_user_tags(name, tags, runas=None):
         tags = ' '.join(tags)
 
     res = __salt__['cmd.run'](
-        'rabbitmqctl set_user_tags {0} {1}'.format(name, tags),
-        python_shell=False,
-        runas=runas)
+        ['rabbitmqctl', 'set_user_tags', name, tags],
+        runas=runas,
+        python_shell=False)
     msg = "Tag(s) set"
     return _format_response(res, msg)
 
@@ -419,9 +467,9 @@ def status(runas=None):
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl status',
-        runas=runas
-    )
+        ['rabbitmqctl', 'status'],
+        runas=runas,
+        python_shell=False)
     return res
 
 
@@ -438,8 +486,9 @@ def cluster_status(runas=None):
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl cluster_status',
-        runas=runas)
+        ['rabbitmqctl', 'cluster_status'],
+        runas=runas,
+        python_shell=False)
 
     return res
 
@@ -454,10 +503,10 @@ def join_cluster(host, user='rabbit', ram_node=None, runas=None):
 
         salt '*' rabbitmq.join_cluster 'rabbit.example.com' 'rabbit'
     '''
+    cmd = ['rabbitmqctl', 'join_cluster']
     if ram_node:
-        cmd = 'rabbitmqctl join_cluster --ram {0}@{1}'.format(user, host)
-    else:
-        cmd = 'rabbitmqctl join_cluster {0}@{1}'.format(user, host)
+        cmd.append('--ram')
+    cmd.append('{0}@{1}'.format(user, host))
 
     if runas is None:
         runas = salt.utils.get_user()
@@ -480,11 +529,10 @@ def stop_app(runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        'rabbitmqctl stop_app',
-        runas=runas)
-
-    return res
+    return __salt__['cmd.run'](
+        ['rabbitmqctl', 'stop_app'],
+        runas=runas,
+        python_shell=False)
 
 
 def start_app(runas=None):
@@ -499,11 +547,10 @@ def start_app(runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        'rabbitmqctl start_app',
-        runas=runas)
-
-    return res
+    return __salt__['cmd.run'](
+        ['rabbitmqctl', 'start_app'],
+        runas=runas,
+        python_shell=False)
 
 
 def reset(runas=None):
@@ -518,11 +565,10 @@ def reset(runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        'rabbitmqctl reset',
-        runas=runas)
-
-    return res
+    return __salt__['cmd.run'](
+        ['rabbitmqctl', 'reset'],
+        runas=runas,
+        python_shell=False)
 
 
 def force_reset(runas=None):
@@ -537,14 +583,13 @@ def force_reset(runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        'rabbitmqctl force_reset',
-        runas=runas)
-
-    return res
+    return __salt__['cmd.run'](
+        ['rabbitmqctl', 'force_reset'],
+        runas=runas,
+        python_shell=False)
 
 
-def list_queues(runas=None, *kwargs):
+def list_queues(runas=None, *args):
     '''
     Returns queue details of the / virtual host
 
@@ -556,15 +601,12 @@ def list_queues(runas=None, *kwargs):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        'rabbitmqctl list_queues {0}'.format(' '.join(list(kwargs))),
-        python_shell=False,
-        runas=runas,
-        )
-    return res
+    cmd = ['rabbitmqctl', 'list_queues']
+    cmd.extend(args)
+    return __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
 
 
-def list_queues_vhost(vhost, runas=None, *kwargs):
+def list_queues_vhost(vhost, runas=None, *args):
     '''
     Returns queue details of specified virtual host. This command will consider
     first parameter as the vhost name and rest will be treated as
@@ -579,15 +621,9 @@ def list_queues_vhost(vhost, runas=None, *kwargs):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        'rabbitmqctl list_queues -p {0} {1}'.format(
-            vhost,
-            ' '.join(list(kwargs))
-            ),
-        python_shell=False,
-        runas=runas,
-        )
-    return res
+    cmd = ['rabbitmqctl', 'list_queues', '-p', vhost]
+    cmd.extend(args)
+    return __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
 
 
 def list_policies(vhost="/", runas=None):
@@ -606,11 +642,12 @@ def list_policies(vhost="/", runas=None):
     ret = {}
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run']('rabbitmqctl list_policies -p {0}'.format(
-                              vhost),
-                              runas=runas)
-    for line in res.splitlines():
-        if '...' not in line and line != '\n':
+    output = __salt__['cmd.run'](
+        ['rabbitmqctl', 'list_policies', '-p', vhost],
+        runas=runas,
+        python_shell=False)
+    for line in salt.utils.itertools.split(output, '\n'):
+        if '...' not in line:
             parts = line.split('\t')
             if len(parts) not in (5, 6):
                 continue
@@ -618,7 +655,8 @@ def list_policies(vhost="/", runas=None):
             if vhost not in ret:
                 ret[vhost] = {}
             ret[vhost][name] = {}
-            # How many fields are there? - 'apply_to' was inserted in position 2 at somepoint
+            # How many fields are there? - 'apply_to' was inserted in position
+            # 2 at some point
             offset = len(parts) - 5
             if len(parts) == 6:
                 ret[vhost][name]['apply_to'] = parts[2]
@@ -645,16 +683,17 @@ def set_policy(vhost, name, pattern, definition, priority=None, runas=None):
     '''
     if runas is None:
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        "rabbitmqctl set_policy -p {0}{1}{2} {3} '{4}' '{5}'".format(
-            vhost,
-            ' --priority ' if priority else '',
-            priority if priority else '',
-            name,
-            pattern,
-            definition.replace("'", '"')),
-        python_shell=False,
-        runas=runas)
+    if isinstance(definition, dict):
+        definition = json.dumps(definition)
+    if not isinstance(definition, six.string_types):
+        raise SaltInvocationError(
+            'The \'definition\' argument must be a dictionary or JSON string'
+        )
+    cmd = ['rabbitmqctl', 'set_policy', '-p', vhost]
+    if priority:
+        cmd.extend(['--priority', priority])
+    cmd.extend([name, pattern, definition])
+    res = __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
     log.debug('Set policy: {0}'.format(res))
     return _format_response(res, 'Set')
 
@@ -674,10 +713,9 @@ def delete_policy(vhost, name, runas=None):
     if runas is None:
         runas = salt.utils.get_user()
     res = __salt__['cmd.run'](
-        'rabbitmqctl clear_policy -p {0} {1}'.format(
-            vhost, name),
-        python_shell=False,
-        runas=runas)
+        ['rabbitmqctl', 'clear_policy', '-p', vhost, name],
+        runas=runas,
+        python_shell=False)
     log.debug('Delete policy: {0}'.format(res))
     return _format_response(res, 'Deleted')
 
@@ -710,10 +748,9 @@ def plugin_is_enabled(name, runas=None):
 
         salt '*' rabbitmq.plugin_is_enabled foo
     '''
-    rabbitmq = _get_rabbitmq_plugin()
-    cmd = '{0} list -m -e'.format(rabbitmq)
     if runas is None:
         runas = salt.utils.get_user()
+    cmd = [_get_rabbitmq_plugin(), 'list', '-m', '-e']
     ret = __salt__['cmd.run_all'](cmd, python_shell=False, runas=runas)
     if ret['retcode'] != 0:
         raise CommandExecutionError(
@@ -732,13 +769,10 @@ def enable_plugin(name, runas=None):
 
         salt '*' rabbitmq.enable_plugin foo
     '''
-    rabbitmq = _get_rabbitmq_plugin()
-    cmd = '{0} enable {1}'.format(rabbitmq, name)
-
     if runas is None:
         runas = salt.utils.get_user()
-    ret = __salt__['cmd.run_all'](cmd, python_shell=False, runas=runas)
-
+    cmd = [_get_rabbitmq_plugin(), 'enable', name]
+    ret = __salt__['cmd.run_all'](cmd, runas=runas, python_shell=False)
     return _format_response(ret, 'Enabled')
 
 
@@ -752,12 +786,8 @@ def disable_plugin(name, runas=None):
 
         salt '*' rabbitmq.disable_plugin foo
     '''
-
-    rabbitmq = _get_rabbitmq_plugin()
-    cmd = '{0} disable {1}'.format(rabbitmq, name)
-
     if runas is None:
         runas = salt.utils.get_user()
-    ret = __salt__['cmd.run_all'](cmd, python_shell=False, runas=runas)
-
+    cmd = [_get_rabbitmq_plugin(), 'disable', name]
+    ret = __salt__['cmd.run_all'](cmd, runas=runas, python_shell=False)
     return _format_response(ret, 'Disabled')
